@@ -784,8 +784,9 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, icalcom
 			prop = icalproperty_new_dtstart (icaltime_from_timet_with_zone (t.tv_sec, (b && *b), zone));
 			if (zone && icaltimezone_get_tzid (zone)) {
 				icalproperty_add_parameter (prop, icalparameter_new_tzid (icaltimezone_get_tzid (zone)));
-				icalcomponent_add_property (ical_comp, prop);
 			}
+
+			icalcomponent_add_property (ical_comp, prop);
 		}
 
 		stream = exchange_mapi_util_find_stream (streams, PROP_TAG(PT_BINARY, 0x825F));
@@ -800,8 +801,9 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, icalcom
 			prop = icalproperty_new_dtend (icaltime_from_timet_with_zone (t.tv_sec, (b && *b), zone));
 			if (zone && icaltimezone_get_tzid (zone)) {
 				icalproperty_add_parameter (prop, icalparameter_new_tzid (icaltimezone_get_tzid (zone)));
-				icalcomponent_add_property (ical_comp, prop);
 			}
+
+			icalcomponent_add_property (ical_comp, prop);
 		}
 
 		ui32 = (const uint32_t *)find_mapi_SPropValue_data(properties, PROP_TAG(PT_LONG, 0x8205));
@@ -875,7 +877,8 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, icalcom
 			e_cal_component_remove_all_alarms (comp);
 
 	} else if (icalcomponent_isa (ical_comp) == ICAL_VTODO_COMPONENT) {
-		const double *complete = 0;
+		const double *complete = NULL;
+		const uint64_t *status = NULL;
 
 		/* NOTE: Exchange tasks are DATE values, not DATE-TIME values, but maybe someday, we could expect Exchange to support it;) */
 		if (get_mapi_SPropValue_array_date_timeval (&t, properties, PROP_TAG(PT_SYSTIME, 0x8104)) == MAPI_E_SUCCESS)
@@ -883,10 +886,10 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, icalcom
 		if (get_mapi_SPropValue_array_date_timeval (&t, properties, PROP_TAG(PT_SYSTIME, 0x8105)) == MAPI_E_SUCCESS)
 			icalcomponent_set_due (ical_comp, icaltime_from_timet_with_zone (t.tv_sec, 1, default_zone));
 
-		ui32 = (const uint32_t *)find_mapi_SPropValue_data(properties, PROP_TAG(PT_LONG, 0x8101));
-		if (ui32) {
-			icalcomponent_set_status (ical_comp, get_taskstatus_from_prop(*ui32));
-			if (*ui32 == olTaskComplete
+		status = (const uint64_t *)find_mapi_SPropValue_data(properties, PROP_TAG(PT_LONG, 0x8101));
+		if (status) {
+			icalcomponent_set_status (ical_comp, get_taskstatus_from_prop(*status));
+			if (*status == olTaskComplete
 			&& get_mapi_SPropValue_array_date_timeval (&t, properties, PROP_TAG(PT_SYSTIME, 0x810F)) == MAPI_E_SUCCESS) {
 				prop = icalproperty_new_completed (icaltime_from_timet_with_zone (t.tv_sec, 1, default_zone));
 				icalcomponent_add_property (ical_comp, prop);
@@ -895,7 +898,7 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, icalcom
 
 		complete = (const double *)find_mapi_SPropValue_data(properties, PROP_TAG(PT_DOUBLE, 0x8102));
 		if (complete) {
-			prop = icalproperty_new_percentcomplete ((gint)(*complete * 100));
+			prop = icalproperty_new_percentcomplete ((gint)(*complete * 100 + 1e-9));
 			icalcomponent_add_property (ical_comp, prop);
 		}
 
@@ -956,315 +959,6 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, icalcom
 
 #define TEMP_ATTACH_STORE ".evolution/cache/tmp"
 
-static void
-change_partstat (ECalComponent *comp, const gchar *att, const gchar *sentby, icalparameter_partstat partstat)
-{
-	icalcomponent *icalcomp = e_cal_component_get_icalcomponent (comp);
-	icalproperty *attendee;
-	gboolean found = FALSE;
-
-	attendee = icalcomponent_get_first_property (icalcomp, ICAL_ATTENDEE_PROPERTY);
-	while (attendee) {
-		const gchar *value = icalproperty_get_attendee (attendee);
-		if (!g_ascii_strcasecmp (value, att)) {
-			icalparameter *param = icalparameter_new_partstat (partstat);
-			icalproperty_set_parameter (attendee, param);
-			if (g_ascii_strcasecmp(att, sentby)) {
-				icalparameter *sentby_param = icalparameter_new_sentby (sentby);
-				icalproperty_set_parameter (attendee, sentby_param);
-			}
-			found = TRUE;
-			break;
-		}
-		attendee = icalcomponent_get_next_property (icalcomp, ICAL_ATTENDEE_PROPERTY);
-	}
-
-	if (found) {
-		icalproperty *prop = icalproperty_new_x ("1");
-		icalproperty_set_x_name (prop, "X-EVOLUTION-IS-REPLY");
-		icalcomponent_add_property (icalcomp, prop);
-	}
-
-	e_cal_component_set_icalcomponent (comp, icalcomp);
-}
-
-static void
-remove_other_attendees (ECalComponent *comp, const gchar *att)
-{
-	icalcomponent *icalcomp = e_cal_component_get_icalcomponent (comp);
-	icalproperty *attendee;
-
-	attendee = icalcomponent_get_first_property (icalcomp, ICAL_ATTENDEE_PROPERTY);
-	while (attendee) {
-		const gchar *value = icalproperty_get_attendee (attendee);
-		if (g_ascii_strcasecmp (value, att))
-			icalcomponent_remove_property (icalcomp, attendee);
-
-		attendee = icalcomponent_get_next_property (icalcomp, ICAL_ATTENDEE_PROPERTY);
-	}
-
-	e_cal_component_set_icalcomponent (comp, icalcomp);
-}
-
-static gboolean
-fetch_server_data_cb (FetchItemsCallbackData *item_data, gpointer data)
-{
-	struct mapi_SPropValue_array *properties = item_data->properties;
-	const mapi_id_t mid = item_data->mid;
-	GSList *streams = item_data->streams;
-	GSList *recipients = item_data->recipients;
-	GSList *attachments = item_data->attachments;
-
-	icalcomponent_kind kind = ICAL_VEVENT_COMPONENT;
-	gchar *filename = g_build_filename (g_get_home_dir (), TEMP_ATTACH_STORE, NULL);
-	gchar *fileuri = g_filename_to_uri (filename, NULL, NULL);
-	gchar *smid = exchange_mapi_util_mapi_id_to_string (mid);
-	ECalComponent *comp = exchange_mapi_cal_util_mapi_props_to_comp (item_data->conn, kind, smid, properties, streams, recipients, attachments, fileuri, NULL);
-	struct cal_cbdata *cbdata = (struct cal_cbdata *)(data);
-	const uint32_t *ui32;
-
-	ui32 = (const uint32_t *)find_mapi_SPropValue_data(properties, PR_OWNER_APPT_ID);
-	cbdata->appt_id = ui32 ? *ui32 : 0;
-	ui32 = (const uint32_t *)find_mapi_SPropValue_data(properties, PROP_TAG(PT_LONG, 0x8201));
-	cbdata->appt_seq = ui32 ? *ui32 : 0;
-	cbdata->username = exchange_mapi_util_find_array_propval (properties, PR_SENT_REPRESENTING_NAME_UNICODE);
-	cbdata->useridtype = exchange_mapi_util_find_array_propval (properties, PR_SENT_REPRESENTING_ADDRTYPE_UNICODE);
-	cbdata->userid = exchange_mapi_util_find_array_propval (properties, PR_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE);
-	cbdata->ownername = exchange_mapi_util_find_array_propval (properties, PR_SENDER_NAME_UNICODE);
-	cbdata->owneridtype = exchange_mapi_util_find_array_propval (properties, PR_SENDER_ADDRTYPE_UNICODE);
-	cbdata->ownerid = exchange_mapi_util_find_array_propval (properties, PR_SENDER_EMAIL_ADDRESS_UNICODE);
-
-	cbdata->comp = comp;
-
-	exchange_mapi_util_free_recipient_list (&recipients);
-
-	g_free (smid);
-	g_free (fileuri);
-	g_free (filename);
-
-	return TRUE;
-}
-
-static void
-fetch_server_data (ExchangeMapiConnection *conn, mapi_id_t mid, struct cal_cbdata *cbd)
-{
-	icalcomponent_kind kind = ICAL_VEVENT_COMPONENT;
-	mapi_id_t fid;
-
-	fid = exchange_mapi_connection_get_default_folder_id (conn, olFolderCalendar);
-
-	exchange_mapi_connection_fetch_item (conn, fid, mid,
-					exchange_mapi_cal_utils_get_props_cb, GINT_TO_POINTER (kind),
-					fetch_server_data_cb, cbd,
-					MAPI_OPTIONS_FETCH_RECIPIENTS | MAPI_OPTIONS_FETCH_GENERIC_STREAMS);
-
-}
-
-static ECalComponent *
-update_attendee_status (ExchangeMapiConnection *conn, struct mapi_SPropValue_array *properties, mapi_id_t mid)
-{
-	const gchar *att, *att_sentby, *addrtype;
-	icalparameter_partstat partstat = ICAL_PARTSTAT_NONE;
-	const gchar *state = (const gchar *) exchange_mapi_util_find_array_propval (properties, PR_MESSAGE_CLASS);
-	struct cal_cbdata cbdata = { 0 };
-	gchar *matt, *matt_sentby;
-	uint32_t cur_seq;
-	const uint32_t *ui32;
-
-	if (!(state && *state))
-		return NULL;
-
-	if (!g_ascii_strcasecmp (state, IPM_SCHEDULE_MEETING_RESP_POS))
-		partstat = ICAL_PARTSTAT_ACCEPTED;
-	else if (!g_ascii_strcasecmp (state, IPM_SCHEDULE_MEETING_RESP_TENT))
-		partstat = ICAL_PARTSTAT_TENTATIVE;
-	else if (!g_ascii_strcasecmp (state, IPM_SCHEDULE_MEETING_RESP_NEG))
-		partstat = ICAL_PARTSTAT_DECLINED;
-	else
-		return NULL;
-
-	fetch_server_data (conn, mid, &cbdata);
-
-	att = exchange_mapi_util_find_array_propval (properties, PR_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE);
-	addrtype = exchange_mapi_util_find_array_propval (properties, PR_SENT_REPRESENTING_ADDRTYPE_UNICODE);
-	if (addrtype && !g_ascii_strcasecmp (addrtype, "EX"))
-		att = exchange_mapi_connection_ex_to_smtp (conn, att);
-
-	att_sentby = exchange_mapi_util_find_array_propval (properties, PR_SENDER_EMAIL_ADDRESS_UNICODE);
-	addrtype = exchange_mapi_util_find_array_propval (properties, PR_SENDER_ADDRTYPE_UNICODE);
-	if (addrtype && !g_ascii_strcasecmp (addrtype, "EX"))
-		att_sentby = exchange_mapi_connection_ex_to_smtp (conn, att_sentby);
-
-	matt = g_strdup_printf ("MAILTO:%s", att);
-	matt_sentby = g_strdup_printf ("MAILTO:%s", att_sentby);
-
-	change_partstat (cbdata.comp, matt, matt_sentby, partstat);
-
-	ui32 = (const uint32_t *) find_mapi_SPropValue_data(properties, PROP_TAG(PT_LONG, 0x8201));
-	cur_seq = ui32 ? *ui32 : 0;
-
-	if (cbdata.appt_seq == cur_seq) {
-
-/*
- * The itip-formatter provides an option to update the attendee's status.
- * Hence, we need not update the server straight away.
- */
-#if 0
-		gchar *filename = g_build_filename (g_get_home_dir (), TEMP_ATTACH_STORE, NULL);
-		gchar *fileuri = g_filename_to_uri (filename, NULL, NULL);
-		GSList *attachments = NULL, *recipients = NULL, *streams = NULL;
-
-		if (e_cal_component_has_attachments (cbdata.comp))
-			exchange_mapi_cal_util_fetch_attachments (cbdata.comp, &attachments, fileuri);
-
-		if (e_cal_component_has_attendees (cbdata.comp))
-			exchange_mapi_cal_util_fetch_recipients (cbdata.comp, &recipients);
-
-		cbdata.kind = kind;
-		cbdata.meeting_type = (recipients != NULL) ? MEETING_OBJECT : NOT_A_MEETING;
-		cbdata.resp = (recipients != NULL) ? olResponseOrganized : olResponseNone;
-		cbdata.msgflags = MSGFLAG_READ;
-		cbdata.is_modify = TRUE;
-		cbdata.cleanglobalid = (const struct SBinary *)find_mapi_SPropValue_data(properties, PROP_TAG(PT_BINARY, 0x0023));
-		cbdata.globalid = (const struct SBinary *)find_mapi_SPropValue_data(properties, PROP_TAG(PT_BINARY, 0x0003));
-		cbdata.get_timezone = NULL;
-		cbdata.get_tz_data = NULL;
-
-		status = exchange_mapi_modify_item (conn, olFolderCalendar, fid, mid,
-				exchange_mapi_cal_utils_write_props_cb, &cbdata,
-				recipients, attachments, streams, MAPI_OPTIONS_DONT_SUBMIT);
-		g_free (cbdata.props);
-
-		exchange_mapi_util_free_recipient_list (&recipients);
-		exchange_mapi_util_free_attachment_list (&attachments);
-		g_free (fileuri);
-		g_free (filename);
-#endif
-
-		/* remove the other attendees so not to confuse itip-formatter */
-		remove_other_attendees (cbdata.comp, matt);
-	} else {
-		g_free (cbdata.props);
-		g_object_unref (cbdata.comp);
-		cbdata.comp = NULL;
-	}
-
-	g_free (matt);
-	g_free (matt_sentby);
-
-	return cbdata.comp;
-}
-
-static void
-update_server_object (ExchangeMapiConnection *conn, struct mapi_SPropValue_array *properties, GSList *attachments, ECalComponent *comp, mapi_id_t *mid)
-{
-	const uint32_t *ui32 = NULL;
-	uint32_t cur_seq;
-	mapi_id_t fid;
-	gboolean create_new = TRUE;
-
-	fid = exchange_mapi_connection_get_default_folder_id (conn, olFolderCalendar);
-
-	ui32 = (const uint32_t *) find_mapi_SPropValue_data(properties, PROP_TAG(PT_LONG, 0x8201));
-	cur_seq = ui32 ? *ui32 : 0;
-
-	if (*mid) {
-		struct cal_cbdata server_cbd = {0};
-		fetch_server_data (conn, *mid, &server_cbd);
-
-		if (cur_seq > server_cbd.appt_seq) {
-			struct id_list idlist;
-			GSList *ids = NULL;
-
-			idlist.id = *mid;
-			ids = g_slist_append (ids, &idlist);
-
-			exchange_mapi_connection_remove_items (conn, olFolderCalendar, fid, ids);
-			g_slist_free (ids);
-		} else
-			create_new = FALSE;
-
-		if (server_cbd.comp)
-			g_object_unref (server_cbd.comp);
-		g_free (server_cbd.props);
-	}
-
-	if (create_new) {
-		struct cal_cbdata cbdata = { 0 };
-		GSList *myrecipients = NULL;
-		GSList *myattachments = NULL;
-		icalcomponent_kind kind = icalcomponent_isa (e_cal_component_get_icalcomponent(comp));
-
-		cbdata.kind = kind;
-		cbdata.comp = comp;
-		cbdata.username = (const gchar *) exchange_mapi_util_find_array_propval (properties, PR_SENDER_NAME_UNICODE);
-		cbdata.useridtype = (const gchar *) exchange_mapi_util_find_array_propval (properties, PR_SENDER_ADDRTYPE_UNICODE);
-		cbdata.userid = (const gchar *) exchange_mapi_util_find_array_propval (properties, PR_SENDER_EMAIL_ADDRESS_UNICODE);
-		cbdata.ownername = (const gchar *) exchange_mapi_util_find_array_propval (properties, PR_SENT_REPRESENTING_NAME_UNICODE);
-		cbdata.owneridtype = (const gchar *) exchange_mapi_util_find_array_propval (properties, PR_SENT_REPRESENTING_ADDRTYPE_UNICODE);
-		cbdata.ownerid = (const gchar *) exchange_mapi_util_find_array_propval (properties, PR_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE);
-		cbdata.is_modify = FALSE;
-		cbdata.msgflags = MSGFLAG_READ;
-		cbdata.meeting_type = MEETING_REQUEST_RCVD;
-		cbdata.resp = olResponseNone;
-		cbdata.appt_seq = (*(const uint32_t *)find_mapi_SPropValue_data(properties, PROP_TAG(PT_LONG, 0x8201)));
-		cbdata.appt_id = (*(const uint32_t *)find_mapi_SPropValue_data(properties, PR_OWNER_APPT_ID));
-		cbdata.globalid = (const struct Binary_r *)find_mapi_SPropValue_data(properties, PROP_TAG(PT_BINARY, 0x0003));
-		cbdata.cleanglobalid = (const struct Binary_r *)find_mapi_SPropValue_data(properties, PROP_TAG(PT_BINARY, 0x0023));
-		cbdata.get_timezone = NULL;
-		cbdata.get_tz_data = NULL;
-
-		exchange_mapi_cal_util_fetch_recipients (comp, &myrecipients);
-		myattachments = attachments;
-		*mid = exchange_mapi_connection_create_item (conn, olFolderCalendar, 0,
-					exchange_mapi_cal_utils_write_props_cb, &cbdata,
-					myrecipients, myattachments, NULL, MAPI_OPTIONS_DONT_SUBMIT);
-		g_free (cbdata.props);
-		exchange_mapi_util_free_recipient_list (&myrecipients);
-	}
-}
-
-static void
-check_server_for_object (ExchangeMapiConnection *conn, struct mapi_SPropValue_array *properties, mapi_id_t *mid)
-{
-	struct mapi_SRestriction res;
-	struct SPropValue sprop;
-	const struct SBinary *sb;
-	uint32_t proptag = 0x0;
-	GSList *ids = NULL, *l;
-	mapi_id_t fid;
-
-	*mid = 0;
-
-	fid = exchange_mapi_connection_get_default_folder_id (conn, olFolderCalendar);
-
-	proptag = exchange_mapi_connection_resolve_named_prop (conn, fid, PidLidCleanGlobalObjectId);
-
-	res.rt = RES_PROPERTY;
-	res.res.resProperty.relop = RELOP_EQ;
-	res.res.resProperty.ulPropTag = proptag;
-
-	sb = (const struct SBinary *)find_mapi_SPropValue_data(properties, proptag);
-	if (!sb)
-		sb = find_mapi_SPropValue_data(properties, PidLidCleanGlobalObjectId);
-
-	set_SPropValue_proptag (&sprop, proptag, (gconstpointer ) sb);
-	cast_mapi_SPropValue (&(res.res.resProperty.lpProp), &sprop);
-
-	ids = exchange_mapi_connection_check_restriction (conn, fid, &res);
-
-	if (ids && g_slist_length(ids) == 1) {
-		struct id_list *idlist = (struct id_list *)(ids->data);
-		*mid = idlist->id;
-	} else  {
-		/* FIXME: what to do here? */
-	}
-
-	for (l = ids; l; l = l->next)
-		g_free(l->data);
-	g_slist_free(l);
-}
-
 struct fetch_camel_cal_data {
 	icalcomponent_kind kind;
 	icalproperty_method method;
@@ -1278,7 +972,7 @@ fetch_camel_cal_comp_cb (FetchItemsCallbackData *item_data, gpointer data)
 	ECalComponent *comp = NULL;
 	mapi_id_t mid = 0;
 	icalcomponent *icalcomp = NULL;
-	gchar *str = NULL, *smid = NULL, *tmp, *filename, *fileuri;
+	gchar *str = NULL, *smid = NULL, *filename, *fileuri;
 
 	g_return_val_if_fail (item_data != NULL, FALSE);
 	g_return_val_if_fail (fccd != NULL, FALSE);
@@ -1286,38 +980,19 @@ fetch_camel_cal_comp_cb (FetchItemsCallbackData *item_data, gpointer data)
 	filename = g_build_filename (g_get_home_dir (), TEMP_ATTACH_STORE, NULL);
 	fileuri = g_filename_to_uri (filename, NULL, NULL);
 
-	check_server_for_object (item_data->conn, item_data->properties, &mid);
-
-	if (fccd->method == ICAL_METHOD_REPLY) {
-		if (mid) {
-			comp = update_attendee_status (item_data->conn, item_data->properties, mid);
-			set_attachments_to_cal_component (comp, item_data->attachments, fileuri);
-		}
-	} else if (fccd->method == ICAL_METHOD_CANCEL) {
-		if (mid) {
-			struct cal_cbdata server_cbd = { 0 };
-			fetch_server_data (item_data->conn, mid, &server_cbd);
-			comp = server_cbd.comp;
-			set_attachments_to_cal_component (comp, item_data->attachments, fileuri);
-
-			g_free (server_cbd.props);
-		}
-	} else if (fccd->method == ICAL_METHOD_REQUEST) {
+	if (!comp) {
+		/* read component from a mail, if not found in the calendar */
 		if (mid)
 			smid = exchange_mapi_util_mapi_id_to_string (mid);
+		else if (item_data->mid)
+			smid = exchange_mapi_util_mapi_id_to_string (item_data->mid);
 		else
 			smid = e_cal_component_gen_uid();
-
 		comp = exchange_mapi_cal_util_mapi_props_to_comp (item_data->conn, fccd->kind, smid,
 							item_data->properties, item_data->streams, item_data->recipients,
 							NULL, NULL, NULL);
 		set_attachments_to_cal_component (comp, item_data->attachments, fileuri);
-
-		update_server_object (item_data->conn, item_data->properties, item_data->attachments, comp, &mid);
-
-		tmp = exchange_mapi_util_mapi_id_to_string (mid);
-		e_cal_component_set_uid (comp, tmp);
-		g_free (tmp);
+		e_cal_component_set_uid (comp, smid);
 		g_free (smid);
 	}
 
@@ -2105,21 +1780,259 @@ exchange_mapi_cal_util_get_new_appt_id (ExchangeMapiConnection *conn, mapi_id_t
 		id = g_random_int ();
 		if (id) {
 			GSList *ids = NULL;
+			TALLOC_CTX *mem_ctx = talloc_init ("ExchangeMAPI_get_new_appt_id");
+
 			set_SPropValue_proptag (&sprop, PR_OWNER_APPT_ID, (gconstpointer ) &id);
-			cast_mapi_SPropValue (&(res.res.resProperty.lpProp), &sprop);
-			ids = exchange_mapi_connection_check_restriction (conn, fid, &res);
+			cast_mapi_SPropValue (
+				#ifdef HAVE_MEMCTX_ON_CAST_MAPI_SPROPVALUE
+				mem_ctx,
+				#endif
+				&(res.res.resProperty.lpProp), &sprop);
+			ids = exchange_mapi_connection_check_restriction (conn, fid, 0, &res);
 			if (ids) {
 				GSList *l;
 				for (l = ids; l; l = l->next)
 					g_free (l->data);
 			} else
 				found = TRUE;
+
+			talloc_free (mem_ctx);
 		}
 	};
 
 	return id;
 }
 
+static time_t
+mapi_get_date_from_string (const gchar *dtstring)
+{
+	time_t t = 0;
+	GTimeVal t_val;
+
+	g_return_val_if_fail (dtstring != NULL, 0);
+
+	if (g_time_val_from_iso8601 (dtstring, &t_val)) {
+		t = (time_t) t_val.tv_sec;
+	} else if (strlen (dtstring) == 8) {
+		/* It might be a date value */
+		GDate date;
+		struct tm tt;
+		guint16 year;
+		guint month;
+		guint8 day;
+
+		g_date_clear (&date, 1);
+#define digit_at(x,y) (x[y] - '0')
+		year = digit_at (dtstring, 0) * 1000
+			+ digit_at (dtstring, 1) * 100
+			+ digit_at (dtstring, 2) * 10
+			+ digit_at (dtstring, 3);
+		month = digit_at (dtstring, 4) * 10 + digit_at (dtstring, 5);
+		day = digit_at (dtstring, 6) * 10 + digit_at (dtstring, 7);
+
+		g_date_set_year (&date, year);
+		g_date_set_month (&date, month);
+		g_date_set_day (&date, day);
+
+		g_date_to_struct_tm (&date, &tt);
+		t = mktime (&tt);
+
+	} else
+		g_warning ("Could not parse the string \n");
+
+        return t;
+}
+
+static void
+populate_freebusy_data (struct Binary_r *bin, uint32_t month, uint32_t year, GList **freebusy, const gchar *accept_type, ECalComponent *comp)
+{
+	uint16_t	event_start;
+	uint16_t	event_end;
+	uint32_t	i;
+	uint32_t       	hour;
+	uint32_t	day;
+	const char	*month_name;
+	uint32_t	minutes;
+	uint32_t	real_month;
+	gchar *date_string = NULL;
+	gchar *start = NULL, *end = NULL;
+	time_t start_date, end_date;
+	icalcomponent *icalcomp = NULL;
+
+	if (!bin)
+		return;
+	/* bin.cb must be a multiple of 4 */
+	if (bin->cb % 4)
+		return;
+
+	year = mapidump_freebusy_year(month, year);
+	month_name = mapidump_freebusy_month(month, year);
+	if (!month_name)
+		return;
+
+	for (i = 0; i < bin->cb; i+= 4) {
+		event_start = (bin->lpb[i + 1] << 8) | bin->lpb[i];
+		event_end = (bin->lpb[i + 3] << 8) | bin->lpb[i + 2];
+
+		for (hour = 0; hour < 24; hour++) {
+			if (!(((event_start - (60 * hour)) % 1440) && (((event_start - (60 * hour)) % 1440) - 30))) {
+				struct icalperiodtype ipt;
+				icalproperty *icalprop;
+				icaltimetype itt;
+
+				day = ((event_start - (60 * hour)) / 1440) + 1;
+				minutes = (event_start - (60 * hour)) % 1440;
+				real_month = month - (year * 16);
+				
+				date_string = g_strdup_printf ("%.2u-%.2u-%.2u", year, real_month, day);
+				start = g_strdup_printf ("%sT%.2u:%.2u:00Z", date_string, hour + daylight, minutes);
+				g_free (date_string);
+
+				day = ((event_end - (60 * hour)) / 1440) + 1;
+				minutes = (event_end - (60 * hour)) % 1440;
+
+				if (minutes >= 60) {
+					hour += minutes / 60;
+					minutes %= 60;
+				}
+
+				date_string = g_strdup_printf ("%.2u-%.2u-%.2u", year, real_month, day);
+				end = g_strdup_printf ("%sT%.2u:%.2u:00Z", date_string, hour + daylight, minutes);
+				g_free (date_string);
+
+				start_date = mapi_get_date_from_string (start);
+				end_date = mapi_get_date_from_string (end);
+
+				memset (&ipt, 0, sizeof (struct icalperiodtype));
+				
+				itt = icaltime_from_timet_with_zone (start_date, 0, icaltimezone_get_utc_timezone ());
+				ipt.start = itt;
+
+				itt = icaltime_from_timet_with_zone (end_date, 0, icaltimezone_get_utc_timezone ());
+				ipt.end = itt;
+			
+				icalcomp = e_cal_component_get_icalcomponent (comp);
+				icalprop = icalproperty_new_freebusy (ipt);
+
+				if (!strcmp (accept_type, "Busy"))
+					icalproperty_set_parameter_from_string (icalprop, "FBTYPE", "BUSY");
+				else if (!strcmp (accept_type, "Tentative"))
+					icalproperty_set_parameter_from_string (icalprop, "FBTYPE", "BUSY-TENTATIVE");
+				else if (!strcmp (accept_type, "OutOfOffice"))
+					icalproperty_set_parameter_from_string (icalprop, "FBTYPE", "BUSY-UNAVAILABLE");
+
+				icalcomponent_add_property(icalcomp, icalprop);
+				g_free (start);
+				g_free (end);
+			}
+		}
+	}
+}
+
+gboolean
+exchange_mapi_cal_utils_get_free_busy_data (ExchangeMapiConnection *conn, GList *users, time_t start, time_t end, GList **freebusy)
+{
+	struct SRow		aRow;
+	enum MAPISTATUS		retval;
+	uint32_t		i;
+	mapi_object_t           obj_store;
+	GList *l;
+
+	const uint32_t			*publish_start;
+	const struct LongArray_r       	*busy_months;
+	const struct BinaryArray_r	*busy_events;
+	const struct LongArray_r       	*tentative_months;
+	const struct BinaryArray_r	*tentative_events;
+	const struct LongArray_r       	*oof_months;
+	const struct BinaryArray_r	*oof_events;
+	uint32_t			year;
+	uint32_t			event_year;
+
+	gchar *name = NULL;
+	ECalComponent *comp;
+	ECalComponentAttendee attendee;
+	GSList *attendee_list = NULL;
+	icalcomponent *icalcomp = NULL;
+	icaltimetype start_time, end_time;
+	icaltimezone *default_zone = NULL;
+
+	exchange_mapi_connection_get_public_folder (conn, &obj_store);
+	
+	for ( l = users; l != NULL; l = g_list_next (l)) {
+		retval = GetUserFreeBusyData (&obj_store, (const gchar *)l->data, &aRow);
+
+		if (retval != MAPI_E_SUCCESS) return false;
+
+		/* Step 2. Dump properties */
+		publish_start = (const uint32_t *) find_SPropValue_data(&aRow, PR_FREEBUSY_START_RANGE);
+		busy_months = (const struct LongArray_r *) find_SPropValue_data(&aRow, PR_FREEBUSY_BUSY_MONTHS);
+		busy_events = (const struct BinaryArray_r *) find_SPropValue_data(&aRow, PR_FREEBUSY_BUSY_EVENTS);
+		tentative_months = (const struct LongArray_r *) find_SPropValue_data(&aRow, PR_FREEBUSY_TENTATIVE_MONTHS);
+		tentative_events = (const struct BinaryArray_r *) find_SPropValue_data(&aRow, PR_FREEBUSY_TENTATIVE_EVENTS);
+		oof_months = (const struct LongArray_r *) find_SPropValue_data(&aRow, PR_FREEBUSY_OOF_MONTHS);
+		oof_events = (const struct BinaryArray_r *) find_SPropValue_data(&aRow, PR_FREEBUSY_OOF_EVENTS);
+
+		year = GetFreeBusyYear(publish_start);
+
+		comp = e_cal_component_new ();
+		e_cal_component_set_new_vtype (comp, E_CAL_COMPONENT_FREEBUSY);
+		e_cal_component_commit_sequence (comp);
+		icalcomp = e_cal_component_get_icalcomponent (comp);
+
+		start_time = icaltime_from_timet_with_zone (start, 0, default_zone ? default_zone : NULL);
+		end_time = icaltime_from_timet_with_zone (end, 0, default_zone ? default_zone : NULL);
+		icalcomponent_set_dtstart (icalcomp, start_time);
+		icalcomponent_set_dtend (icalcomp, end_time);
+
+		memset (&attendee, 0, sizeof (ECalComponentAttendee));
+		if (name)
+			attendee.cn = name;
+		if (l->data)
+			attendee.value = l->data;
+
+		attendee.cutype = ICAL_CUTYPE_INDIVIDUAL;
+		attendee.role = ICAL_ROLE_REQPARTICIPANT;
+		attendee.status = ICAL_PARTSTAT_NEEDSACTION;
+
+		attendee_list = g_slist_append (attendee_list, &attendee);
+
+		e_cal_component_set_attendee_list (comp, attendee_list);
+		g_slist_free (attendee_list);
+		g_free ((gchar *) name);
+		g_free ((gchar *) l->data);
+
+		if (busy_months && ((*(const uint32_t *) busy_months) != MAPI_E_NOT_FOUND) &&
+		    busy_events && ((*(const uint32_t *) busy_events) != MAPI_E_NOT_FOUND)) {
+			for (i = 0; i < busy_months->cValues; i++) {
+				event_year = mapidump_freebusy_year(busy_months->lpl[i], year);
+				populate_freebusy_data (&busy_events->lpbin[i], busy_months->lpl[i], event_year, freebusy, "Busy", comp);
+			}
+		}
+		
+		if (tentative_months && ((*(const uint32_t *) tentative_months) != MAPI_E_NOT_FOUND) &&
+		    tentative_events && ((*(const uint32_t *) tentative_events) != MAPI_E_NOT_FOUND)) {
+			for (i = 0; i < tentative_months->cValues; i++) {
+				event_year = mapidump_freebusy_year(tentative_months->lpl[i], year);
+				populate_freebusy_data (&tentative_events->lpbin[i], tentative_months->lpl[i], event_year, freebusy, "Tentative", comp);
+			}
+		}
+
+		if (oof_months && ((*(const uint32_t *) oof_months) != MAPI_E_NOT_FOUND) &&
+		    oof_events && ((*(const uint32_t *) oof_events) != MAPI_E_NOT_FOUND)) {
+			for (i = 0; i < oof_months->cValues; i++) {
+				event_year = mapidump_freebusy_year(oof_months->lpl[i], year);
+				populate_freebusy_data (&oof_events->lpbin[i], oof_months->lpl[i], event_year, freebusy, "OutOfOffice", comp);
+			}
+		}
+
+		e_cal_component_commit_sequence (comp);
+		*freebusy = g_list_append (*freebusy, e_cal_component_get_as_string (comp));
+//		g_object_unref (comp);
+		MAPIFreeBuffer(aRow.lpProps);
+	}
+	return TRUE;
+}
+
 /* beware, the 'data' pointer is an integer of the event kind */
 gboolean
 exchange_mapi_cal_utils_get_props_cb (ExchangeMapiConnection *conn, mapi_id_t fid, TALLOC_CTX *mem_ctx, struct SPropTagArray *props, gpointer data)
